@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2003  Red Hat, Inc.
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include "utils.h"
 #include "signals.h"
 #include "expirelist.h"
+#include "selinux.h"
 #include <dbus/dbus-list.h>
 #include <dbus/dbus-hash.h>
 #include <dbus/dbus-timeout.h>
@@ -75,6 +76,8 @@ typedef struct
   DBusPreallocatedSend *oom_preallocated;
   BusClientPolicy *policy;
 
+  BusSELinuxID *selinux_id;
+
   long connection_tv_sec;  /**< Time when we connected (seconds component) */
   long connection_tv_usec; /**< Time when we connected (microsec component) */
   int stamp;               /**< connections->stamp last time we were traversed */
@@ -401,6 +404,9 @@ free_connection_data (void *data)
 
   if (d->policy)
     bus_client_policy_unref (d->policy);
+
+  if (d->selinux_id)
+    bus_selinux_id_unref (d->selinux_id);
   
   dbus_free (d->name);
   
@@ -517,7 +523,6 @@ bus_connections_unref (BusConnections *connections)
 
       _dbus_assert (connections->n_completed == 0);
 
-      _dbus_assert (connections->pending_replies->n_items == 0);
       bus_expire_list_free (connections->pending_replies);
       
       _dbus_loop_remove_timeout (bus_context_get_loop (connections->context),
@@ -540,6 +545,7 @@ bus_connections_setup_connection (BusConnections *connections,
 {
   BusConnectionData *d;
   dbus_bool_t retval;
+  DBusError error;
   
   d = dbus_new0 (BusConnectionData, 1);
   
@@ -563,6 +569,20 @@ bus_connections_setup_connection (BusConnections *connections,
     }
 
   retval = FALSE;
+
+  dbus_error_init (&error);
+  d->selinux_id = bus_selinux_init_connection_id (connection,
+                                                  &error);
+  if (dbus_error_is_set (&error))
+    {
+      /* This is a bit bogus because we pretend all errors
+       * are OOM; this is done because we know that in bus.c
+       * an OOM error disconnects the connection, which is
+       * the same thing we want on any other error.
+       */
+      dbus_error_free (&error);
+      goto out;
+    }
   
   if (!dbus_connection_set_watch_functions (connection,
                                             add_connection_watch,
@@ -640,7 +660,11 @@ bus_connections_setup_connection (BusConnections *connections,
 
  out:
   if (!retval)
-    {      
+    {
+      if (d->selinux_id)
+        bus_selinux_id_unref (d->selinux_id);
+      d->selinux_id = NULL;
+      
       if (!dbus_connection_set_watch_functions (connection,
                                                 NULL, NULL, NULL,
                                                 connection,
@@ -1009,6 +1033,18 @@ bus_connection_get_matchmaker (DBusConnection *connection)
   return bus_context_get_matchmaker (d->connections->context);
 }
 
+BusSELinuxID*
+bus_connection_get_selinux_id (DBusConnection *connection)
+{
+  BusConnectionData *d;
+
+  d = BUS_CONNECTION_DATA (connection);
+
+  _dbus_assert (d != NULL);
+
+  return d->selinux_id;
+}
+
 /**
  * Checks whether the connection is registered with the message bus.
  *
@@ -1544,6 +1580,7 @@ bus_connections_expect_reply (BusConnections  *connections,
   dbus_uint32_t reply_serial;
   DBusList *link;
   CancelPendingReplyData *cprd;
+  int count;
 
   _dbus_assert (will_get_reply != NULL);
   _dbus_assert (will_send_reply != NULL);
@@ -1555,6 +1592,7 @@ bus_connections_expect_reply (BusConnections  *connections,
   reply_serial = dbus_message_get_serial (reply_to_this);
 
   link = _dbus_list_get_first_link (&connections->pending_replies->items);
+  count = 0;
   while (link != NULL)
     {
       pending = link->data;
@@ -1570,8 +1608,17 @@ bus_connections_expect_reply (BusConnections  *connections,
       
       link = _dbus_list_get_next_link (&connections->pending_replies->items,
                                        link);
+      ++count;
     }
   
+  if (count >=
+      bus_context_get_max_replies_per_connection (connections->context))
+    {
+      dbus_set_error (error, DBUS_ERROR_LIMITS_EXCEEDED,
+		      "The maximum number of pending replies per connection has been reached");
+      return FALSE;
+    }
+
   pending = dbus_new0 (BusPendingReply, 1);
   if (pending == NULL)
     {
