@@ -2,7 +2,7 @@
 /* driver.c  Bus client (driver)
  *
  * Copyright (C) 2003 CodeFactory AB
- * Copyright (C) 2003 Red Hat, Inc.
+ * Copyright (C) 2003, 2004 Red Hat, Inc.
  *
  * Licensed under the Academic Free License version 2.1
  * 
@@ -39,56 +39,26 @@ static dbus_bool_t bus_driver_send_welcome_message (DBusConnection *connection,
                                                     DBusError      *error);
 
 dbus_bool_t
-bus_driver_send_service_deleted (const char     *service_name,
-                                 BusTransaction *transaction,
-                                 DBusError      *error)
+bus_driver_send_service_owner_changed (const char     *service_name,
+				       const char     *old_owner,
+				       const char     *new_owner,
+				       BusTransaction *transaction,
+				       DBusError      *error)
 {
   DBusMessage *message;
   dbus_bool_t retval;
+  const char null_service[] = { '\000' };
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
   
-  _dbus_verbose ("sending service deleted: %s\n", service_name);
+  _dbus_verbose ("sending service owner changed: %s [%s -> %s]\n",
+                 service_name, 
+                 old_owner ? old_owner : null_service, 
+                 new_owner ? new_owner : null_service);
 
   message = dbus_message_new_signal (DBUS_PATH_ORG_FREEDESKTOP_DBUS,
                                      DBUS_INTERFACE_ORG_FREEDESKTOP_DBUS,
-                                     "ServiceDeleted");
-  
-  if (message == NULL)
-    {
-      BUS_SET_OOM (error);
-      return FALSE;
-    }
-  
-  if (!dbus_message_set_sender (message, DBUS_SERVICE_ORG_FREEDESKTOP_DBUS) ||
-      !dbus_message_append_args (message,
-                                 DBUS_TYPE_STRING, service_name,
-                                 DBUS_TYPE_INVALID))
-    {
-      dbus_message_unref (message);
-      BUS_SET_OOM (error);
-      return FALSE;
-    }
-
-  retval = bus_dispatch_matches (transaction, NULL, NULL, message, error);
-  dbus_message_unref (message);
-
-  return retval;
-}
-
-dbus_bool_t
-bus_driver_send_service_created (const char     *service_name,
-                                 BusTransaction *transaction,
-                                 DBusError      *error)
-{
-  DBusMessage *message;
-  dbus_bool_t retval;
-
-  _DBUS_ASSERT_ERROR_IS_CLEAR (error);
-  
-  message = dbus_message_new_signal (DBUS_PATH_ORG_FREEDESKTOP_DBUS,
-                                     DBUS_INTERFACE_ORG_FREEDESKTOP_DBUS,
-                                     "ServiceCreated");
+                                     "ServiceOwnerChanged");
   
   if (message == NULL)
     {
@@ -97,25 +67,26 @@ bus_driver_send_service_created (const char     *service_name,
     }
   
   if (!dbus_message_set_sender (message, DBUS_SERVICE_ORG_FREEDESKTOP_DBUS))
-    {
-      dbus_message_unref (message);
-      BUS_SET_OOM (error);
-      return FALSE;
-    }
-  
+    goto oom;
+
   if (!dbus_message_append_args (message,
                                  DBUS_TYPE_STRING, service_name,
+                                 DBUS_TYPE_STRING, old_owner ? old_owner : null_service,
+                                 DBUS_TYPE_STRING, new_owner ? new_owner : null_service,
                                  DBUS_TYPE_INVALID))
-    {
-      dbus_message_unref (message);
-      BUS_SET_OOM (error);
-      return FALSE;
-    }
+    goto oom;
+
+  _dbus_assert (dbus_message_has_signature (message, "sss"));
   
   retval = bus_dispatch_matches (transaction, NULL, NULL, message, error);
   dbus_message_unref (message);
 
   return retval;
+
+ oom:
+  dbus_message_unref (message);
+  BUS_SET_OOM (error);
+  return FALSE;
 }
 
 dbus_bool_t
@@ -278,6 +249,14 @@ bus_driver_handle_hello (DBusConnection *connection,
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
+  if (bus_connection_is_active (connection))
+    {
+      /* We already handled an Hello message for this connection. */
+      dbus_set_error (error, DBUS_ERROR_FAILED,
+                      "Already handled an Hello message");
+      return FALSE;
+    }
+
   /* Note that when these limits are exceeded we don't disconnect the
    * connection; we just sort of leave it hanging there until it times
    * out or disconnects itself or is dropped due to the max number of
@@ -370,6 +349,8 @@ bus_driver_send_welcome_message (DBusConnection *connection,
       return FALSE;
     }
 
+  _dbus_assert (dbus_message_has_signature (welcome, "s"));
+  
   if (!bus_transaction_send_from_driver (transaction, connection, welcome))
     {
       dbus_message_unref (welcome);
@@ -447,7 +428,7 @@ bus_driver_handle_acquire_service (DBusConnection *connection,
   DBusString service_name;
   char *name;
   int service_reply;
-  int flags;
+  dbus_uint32_t flags;
   dbus_bool_t retval;
   BusRegistry *registry;
 
@@ -511,6 +492,7 @@ bus_driver_handle_service_exists (DBusConnection *connection,
   DBusMessage *reply;
   DBusString service_name;
   BusService *service;
+  dbus_bool_t service_exists;
   char *name;
   dbus_bool_t retval;
   BusRegistry *registry;
@@ -525,10 +507,18 @@ bus_driver_handle_service_exists (DBusConnection *connection,
     return FALSE;
 
   retval = FALSE;
+
+  if (strcmp (name, DBUS_SERVICE_ORG_FREEDESKTOP_DBUS) == 0)
+    {
+      service_exists = TRUE;
+    }
+  else
+    {
+      _dbus_string_init_const (&service_name, name);
+      service = bus_registry_lookup (registry, &service_name);
+      service_exists = service != NULL;
+    }
   
-  _dbus_string_init_const (&service_name, name);
-  service = bus_registry_lookup (registry, &service_name);
- 
   reply = dbus_message_new_method_return (message);
   if (reply == NULL)
     {
@@ -537,7 +527,7 @@ bus_driver_handle_service_exists (DBusConnection *connection,
     }
 
   if (!dbus_message_append_args (reply,
-                                 DBUS_TYPE_BOOLEAN, service != NULL,
+                                 DBUS_TYPE_BOOLEAN, service_exists,
                                  0))
     {
       BUS_SET_OOM (error);
@@ -842,7 +832,6 @@ bus_driver_handle_get_connection_unix_user (DBusConnection *connection,
   DBusConnection *conn;
   DBusMessage *reply;
   unsigned long uid;
-  const char *base_name;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
@@ -919,7 +908,6 @@ bus_driver_handle_get_connection_unix_process_id (DBusConnection *connection,
   DBusConnection *conn;
   DBusMessage *reply;
   unsigned long pid;
-  const char *base_name;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
