@@ -164,7 +164,10 @@ bus_registry_ensure (BusRegistry               *registry,
       return NULL;
     }
 
-  if (!bus_driver_send_service_created (service->name, transaction, error))
+  if (!bus_driver_send_service_owner_changed (service->name, 
+					      NULL,
+					      bus_connection_get_name (owner_if_created),
+					      transaction, error))
     {
       bus_service_unref (service);
       return NULL;
@@ -307,7 +310,8 @@ bus_registry_acquire_service (BusRegistry      *registry,
   sid = bus_selinux_id_table_lookup (registry->service_sid_table,
                                      service_name);
 
-  if (!bus_selinux_allows_acquire_service (connection, sid))
+  if (!bus_selinux_allows_acquire_service (connection, sid,
+					   _dbus_string_get_const_data (service_name)))
     {
       dbus_set_error (error, DBUS_ERROR_ACCESS_DENIED,
                       "Connection \"%s\" is not allowed to own the service \"%s\" due "
@@ -315,7 +319,7 @@ bus_registry_acquire_service (BusRegistry      *registry,
                       bus_connection_is_active (connection) ?
                       bus_connection_get_name (connection) :
                       "(inactive)",
-		      service_name);
+                      _dbus_string_get_const_data (service_name));
       goto out;
     }
       
@@ -328,7 +332,7 @@ bus_registry_acquire_service (BusRegistry      *registry,
                       bus_connection_is_active (connection) ?
                       bus_connection_get_name (connection) :
                       "(inactive)",
-		      service_name);
+                      _dbus_string_get_const_data (service_name));
       goto out;
     }
 
@@ -414,17 +418,33 @@ bus_registry_acquire_service (BusRegistry      *registry,
   return retval;
 }
 
-void
-bus_registry_set_service_sid_table (BusRegistry   *registry,
-                                    DBusHashTable *table)
+dbus_bool_t
+bus_registry_set_service_context_table (BusRegistry   *registry,
+					DBusHashTable *table)
 {
-  _dbus_assert (registry->service_sid_table != table);
+  DBusHashTable *new_table;
+  DBusHashIter iter;
+  
+  new_table = bus_selinux_id_table_new ();
+  if (!new_table)
+    return FALSE;
+
+  _dbus_hash_iter_init (table, &iter);
+  while (_dbus_hash_iter_next (&iter))
+    {
+      const char *service = _dbus_hash_iter_get_string_key (&iter);
+      const char *context = _dbus_hash_iter_get_value (&iter);
+
+      if (!bus_selinux_id_table_insert (new_table,
+					service,
+					context))
+	return FALSE;
+    }
   
   if (registry->service_sid_table)
     _dbus_hash_table_unref (registry->service_sid_table);
-
-  registry->service_sid_table = table;
-  _dbus_hash_table_ref (table);
+  registry->service_sid_table = new_table;
+  return TRUE;
 }
 
 static void
@@ -726,20 +746,31 @@ bus_service_remove_owner (BusService     *service,
     }
   else if (_dbus_list_length_is_one (&service->owners))
     {
-      if (!bus_driver_send_service_deleted (service->name,
-                                            transaction, error))
+      if (!bus_driver_send_service_owner_changed (service->name,
+ 						  bus_connection_get_name (owner),
+ 						  NULL,
+ 						  transaction, error))
         return FALSE;
     }
   else
     {
       DBusList *link;
+      DBusConnection *new_owner;
       link = _dbus_list_get_first_link (&service->owners);
       _dbus_assert (link != NULL);
       link = _dbus_list_get_next_link (&service->owners, link);
       _dbus_assert (link != NULL);
 
+      new_owner = link->data;
+
+      if (!bus_driver_send_service_owner_changed (service->name,
+ 						  bus_connection_get_name (owner),
+ 						  bus_connection_get_name (new_owner),
+ 						  transaction, error))
+        return FALSE;
+
       /* This will be our new owner */
-      if (!bus_driver_send_service_acquired (link->data,
+      if (!bus_driver_send_service_acquired (new_owner,
                                              service->name,
                                              transaction,
                                              error))
