@@ -4,7 +4,7 @@
  * Copyright (C) 2003  CodeFactory AB
  * Copyright (C) 2003  Red Hat, Inc.
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,9 +28,9 @@
 #include <dbus/dbus.h>
 
 extern const char bus_no_memory_message[];
-#define BUS_SET_OOM(error) dbus_set_error ((error), DBUS_ERROR_NO_MEMORY, bus_no_memory_message)
+#define BUS_SET_OOM(error) dbus_set_error_const ((error), DBUS_ERROR_NO_MEMORY, bus_no_memory_message)
 
 void        bus_connection_dispatch_all_messages (DBusConnection *connection);
 dbus_bool_t bus_connection_dispatch_one_message  (DBusConnection *connection);
 
-#endif /* BUS_ACTIVATION_H */
+#endif /* BUS_UTILS_H */
