@@ -191,6 +191,7 @@ _dbus_verbose_real (const char *format,
   va_list args;
   static dbus_bool_t verbose = TRUE;
   static dbus_bool_t need_pid = TRUE;
+  int len;
   
   /* things are written a bit oddly here so that
    * in the non-verbose case we just have the one
@@ -207,18 +208,16 @@ _dbus_verbose_real (const char *format,
         return;
     }
 
+  /* Print out pid before the line */
   if (need_pid)
-    {
-      int len;
-      
-      fprintf (stderr, "%lu: ", _dbus_getpid ());
+    fprintf (stderr, "%lu: ", _dbus_getpid ());
 
-      len = strlen (format);
-      if (format[len-1] == '\n')
-        need_pid = TRUE;
-      else
-        need_pid = FALSE;
-    }
+  /* Only print pid again if the next line is a new line */
+  len = strlen (format);
+  if (format[len-1] == '\n')
+    need_pid = TRUE;
+  else
+    need_pid = FALSE;
   
   va_start (args, format);
   vfprintf (stderr, format, args);
@@ -418,6 +417,8 @@ _dbus_header_field_to_string (int header_field)
       return "destination";
     case DBUS_HEADER_FIELD_SENDER:
       return "sender";
+    case DBUS_HEADER_FIELD_SIGNATURE:
+      return "signature";
     default:
       return "unknown";
     }
