@@ -35,7 +35,7 @@
 #include <dbus/dbus-sysdeps.h>
 #include <dbus/dbus-threads.h>
 
-DBUS_BEGIN_DECLS;
+DBUS_BEGIN_DECLS
 
 void _dbus_warn               (const char *format,
                                ...) _DBUS_GNUC_PRINTF (1, 2);
@@ -198,6 +198,8 @@ char**      _dbus_dup_string_array      (const char **array);
 #undef	ABS
 #define ABS(a)	   (((a) < 0) ? -(a) : (a))
 
+#define _DBUS_ISASCII(c) ((c) != '\0' && (((c) & ~0x7f) == 0))
+
 typedef void (* DBusForeachFunction) (void *element,
                                       void *data);
 
@@ -265,10 +267,11 @@ _DBUS_DECLARE_GLOBAL_LOCK (atomic);
 _DBUS_DECLARE_GLOBAL_LOCK (bus);
 _DBUS_DECLARE_GLOBAL_LOCK (shutdown_funcs);
 _DBUS_DECLARE_GLOBAL_LOCK (system_users);
-#define _DBUS_N_GLOBAL_LOCKS (9)
+_DBUS_DECLARE_GLOBAL_LOCK (message_cache);
+#define _DBUS_N_GLOBAL_LOCKS (10)
 
 dbus_bool_t _dbus_threads_init_debug (void);
 
-DBUS_END_DECLS;
+DBUS_END_DECLS
 
 #endif /* DBUS_INTERNALS_H */
