@@ -30,11 +30,11 @@
 #include <dbus/dbus-types.h>
 #include <dbus/dbus-string.h>
 
-DBUS_BEGIN_DECLS;
+DBUS_BEGIN_DECLS
 
 dbus_bool_t _dbus_message_data_load (DBusString       *dest,
                                      const DBusString *filename);
 
-DBUS_END_DECLS;
+DBUS_END_DECLS
 
 #endif /* DBUS_MESSAGE_BUILDER_H */
