@@ -34,7 +34,7 @@
 #include <dbus/dbus-errors.h>
 #include <stdarg.h>
 
-DBUS_BEGIN_DECLS;
+DBUS_BEGIN_DECLS
 
 typedef struct DBusMessage DBusMessage;
 typedef struct DBusMessageIter DBusMessageIter;
@@ -295,7 +295,8 @@ void*       dbus_message_get_data           (DBusMessage      *message,
                                              dbus_int32_t      slot);
 
 int dbus_message_type_from_string (const char *type_str);
+const char * dbus_message_type_to_string (int type);
 
-DBUS_END_DECLS;
+DBUS_END_DECLS
 
 #endif /* DBUS_MESSAGE_H */
