@@ -72,13 +72,13 @@ read_ints (int        fd,
     {
       size_t chunk;
       size_t to_read;
-      
-    again:
 
       to_read = sizeof (int) * n_ints_in_buf - bytes;
 
       if (to_read == 0)
         break;
+
+    again:
       
       chunk = read (fd,
                     ((char*)buf) + bytes,
@@ -128,11 +128,12 @@ read_pid (int        fd,
       size_t chunk;    
       size_t to_read;
       
-    again:
       to_read = sizeof (pid_t) - bytes;
 
       if (to_read == 0)
         break;
+
+    again:
       
       chunk = read (fd,
                     ((char*)buf) + bytes,
@@ -863,7 +864,14 @@ check_babysit_events (pid_t grandchild_pid,
   pid_t ret;
   int status;
   
-  ret = waitpid (grandchild_pid, &status, WNOHANG);
+  do
+    {
+      ret = waitpid (grandchild_pid, &status, WNOHANG);
+      /* The man page says EINTR can't happen with WNOHANG,
+       * but there are reports of it (maybe only with valgrind?)
+       */
+    }
+  while (ret < 0 && errno == EINTR);
 
   if (ret == 0)
     {
