@@ -32,7 +32,7 @@
 
 #include <stdarg.h>
 
-DBUS_BEGIN_DECLS;
+DBUS_BEGIN_DECLS
 
 /**
  * DBusString object
@@ -49,6 +49,16 @@ struct DBusString
   unsigned int dummy8 : 3; /**< placeholder */
 };
 
+#ifdef DBUS_DISABLE_ASSERT
+/* Some simple inlining hacks; the current linker is not smart enough
+ * to inline non-exported symbols across files in the library.
+ * Note that these break type safety (due to the casts)
+ */
+#define _dbus_string_get_length(s) (((DBusString*)(s))->dummy2)
+#define _dbus_string_get_byte(s, i) (((const unsigned char*)(((DBusString*)(s))->dummy1))[(i)])
+#define _dbus_string_get_const_data(s) ((const char*)(((DBusString*)(s))->dummy1))
+#endif
+
 dbus_bool_t   _dbus_string_init                  (DBusString        *str);
 void          _dbus_string_init_const            (DBusString        *str,
                                                   const char        *value);
@@ -60,7 +70,9 @@ dbus_bool_t   _dbus_string_init_preallocated     (DBusString        *str,
 void          _dbus_string_free                  (DBusString        *str);
 void          _dbus_string_lock                  (DBusString        *str);
 char*         _dbus_string_get_data              (DBusString        *str);
+#ifndef _dbus_string_get_const_data
 const char*   _dbus_string_get_const_data        (const DBusString  *str);
+#endif /* _dbus_string_get_const_data */
 char*         _dbus_string_get_data_len          (DBusString        *str,
                                                   int                start,
                                                   int                len);
@@ -70,12 +82,17 @@ const char*   _dbus_string_get_const_data_len    (const DBusString  *str,
 void          _dbus_string_set_byte              (DBusString        *str,
                                                   int                i,
                                                   unsigned char      byte);
+#ifndef _dbus_string_get_byte
 unsigned char _dbus_string_get_byte              (const DBusString  *str,
                                                   int                start);
+#endif /* _dbus_string_get_byte */
 dbus_bool_t   _dbus_string_insert_bytes          (DBusString        *str,
                                                   int                i,
 						  int                n_bytes,
                                                   unsigned char      byte);
+dbus_bool_t   _dbus_string_insert_byte           (DBusString        *str,
+                                                  int                i,
+                                                  unsigned char      byte);
 dbus_bool_t   _dbus_string_steal_data            (DBusString        *str,
                                                   char             **data_return);
 dbus_bool_t   _dbus_string_steal_data_len        (DBusString        *str,
@@ -88,7 +105,13 @@ dbus_bool_t   _dbus_string_copy_data_len         (const DBusString  *str,
                                                   char             **data_return,
                                                   int                start,
                                                   int                len);
+void          _dbus_string_copy_to_buffer        (const DBusString  *str,
+                                                  char              *buffer,
+						  int                len);
+#ifndef _dbus_string_get_length
 int           _dbus_string_get_length            (const DBusString  *str);
+#endif /* !_dbus_string_get_length */
+
 dbus_bool_t   _dbus_string_lengthen              (DBusString        *str,
                                                   int                additional_length);
 void          _dbus_string_shorten               (DBusString        *str,
@@ -97,6 +120,8 @@ dbus_bool_t   _dbus_string_set_length            (DBusString        *str,
                                                   int                length);
 dbus_bool_t   _dbus_string_align_length          (DBusString        *str,
                                                   int                alignment);
+dbus_bool_t   _dbus_string_alloc_space           (DBusString        *str,
+                                                  int                extra_bytes);
 dbus_bool_t   _dbus_string_append                (DBusString        *str,
                                                   const char        *buffer);
 dbus_bool_t   _dbus_string_append_len            (DBusString        *str,
@@ -122,6 +147,15 @@ dbus_bool_t   _dbus_string_append_printf         (DBusString        *str,
 dbus_bool_t   _dbus_string_append_printf_valist  (DBusString        *str,
                                                   const char        *format,
                                                   va_list            args);
+dbus_bool_t   _dbus_string_insert_4_aligned      (DBusString        *str,
+                                                  int                insert_at,
+                                                  const unsigned char octets[4]);
+dbus_bool_t   _dbus_string_insert_8_aligned      (DBusString        *str,
+                                                  int                insert_at,
+                                                  const unsigned char octets[8]);
+dbus_bool_t   _dbus_string_insert_alignment      (DBusString        *str,
+                                                  int               *insert_at,
+                                                  int                alignment);
 void          _dbus_string_delete                (DBusString        *str,
                                                   int                start,
                                                   int                len);
@@ -199,6 +233,11 @@ dbus_bool_t   _dbus_string_equal_c_str           (const DBusString  *a,
 dbus_bool_t   _dbus_string_equal_len             (const DBusString  *a,
                                                   const DBusString  *b,
                                                   int                len);
+dbus_bool_t   _dbus_string_equal_substring       (const DBusString  *a,
+                                                  int                a_start,
+                                                  int                a_len,
+                                                  const DBusString  *b,
+                                                  int                b_start);
 dbus_bool_t   _dbus_string_starts_with_c_str     (const DBusString  *a,
                                                   const char        *c_str);
 dbus_bool_t   _dbus_string_ends_with_c_str       (const DBusString  *a,
@@ -240,9 +279,12 @@ dbus_bool_t   _dbus_string_validate_error_name   (const DBusString  *str,
 dbus_bool_t   _dbus_string_validate_service      (const DBusString  *str,
                                                   int                start,
                                                   int                len);
+dbus_bool_t   _dbus_string_validate_signature    (const DBusString  *str,
+                                                  int                start,
+                                                  int                len);
 void          _dbus_string_zero                  (DBusString        *str);
 
 
-DBUS_END_DECLS;
+DBUS_END_DECLS
 
 #endif /* DBUS_STRING_H */
