@@ -622,6 +622,7 @@ _dbus_connect_tcp_socket (const char     *host,
                       _dbus_error_from_errno (errno),
                       "Failed to lookup hostname: %s",
                       host);
+      close (fd);
       return -1;
     }
   
@@ -695,6 +696,7 @@ _dbus_listen_tcp_socket (const char     *host,
                       _dbus_error_from_errno (errno),
                       "Failed to lookup hostname: %s",
                       host);
+      close (listen_fd);
       return -1;
     }
   
@@ -1564,7 +1566,6 @@ fill_user_info (DBusUserInfo       *info,
   
  failed:
   _DBUS_ASSERT_ERROR_IS_SET (error);
-  _dbus_user_info_free (info);
   return FALSE;
 }
 
@@ -2120,7 +2121,7 @@ _dbus_file_get_contents (DBusString       *str,
     {
       dbus_set_error (error, DBUS_ERROR_FAILED,
                       "File size %lu of \"%s\" is too large.",
-                      filename_c, (unsigned long) sb.st_size);
+                      (unsigned long) sb.st_size, filename_c);
       close (fd);
       return FALSE;
     }
@@ -3018,6 +3019,9 @@ _dbus_full_duplex_pipe (int        *fd1,
   
   *fd1 = fds[0];
   *fd2 = fds[1];
+
+  _dbus_verbose ("full-duplex pipe %d <-> %d\n",
+                 *fd1, *fd2);
   
   return TRUE;  
 #else
@@ -3130,11 +3134,13 @@ _dbus_print_backtrace (void)
  * Does the chdir, fork, setsid, etc. to become a daemon process.
  *
  * @param pidfile #NULL, or pidfile to create
+ * @param print_pid_fd file descriptor to print pid to, or -1 for none
  * @param error return location for errors
  * @returns #FALSE on failure
  */
 dbus_bool_t
 _dbus_become_daemon (const DBusString *pidfile,
+		     int               print_pid_fd,
                      DBusError        *error)
 {
   const char *s;
@@ -3200,6 +3206,42 @@ _dbus_become_daemon (const DBusString *pidfile,
               return FALSE;
             }
         }
+
+      /* Write PID if requested */
+      if (print_pid_fd >= 0)
+	{
+	  DBusString pid;
+	  int bytes;
+	  
+	  if (!_dbus_string_init (&pid))
+	    {
+	      _DBUS_SET_OOM (error);
+              kill (child_pid, SIGTERM);
+	      return FALSE;
+	    }
+	  
+	  if (!_dbus_string_append_int (&pid, _dbus_getpid ()) ||
+	      !_dbus_string_append (&pid, "\n"))
+	    {
+	      _dbus_string_free (&pid);
+	      _DBUS_SET_OOM (error);
+              kill (child_pid, SIGTERM);
+	      return FALSE;
+	    }
+	  
+	  bytes = _dbus_string_get_length (&pid);
+	  if (_dbus_write (print_pid_fd, &pid, 0, bytes) != bytes)
+	    {
+	      dbus_set_error (error, DBUS_ERROR_FAILED,
+			      "Printing message bus PID: %s\n",
+			      _dbus_strerror (errno));
+	      _dbus_string_free (&pid);
+              kill (child_pid, SIGTERM);
+	      return FALSE;
+	    }
+	  
+	  _dbus_string_free (&pid);
+	}
       _dbus_verbose ("parent exiting\n");
       _exit (0);
       break;
@@ -3281,6 +3323,16 @@ _dbus_change_identity  (dbus_uid_t     uid,
                         dbus_gid_t     gid,
                         DBusError     *error)
 {
+  /* setgroups() only works if we are a privileged process,
+   * so we don't return error on failure; the only possible
+   * failure is that we don't have perms to do it.
+   * FIXME not sure this is right, maybe if setuid()
+   * is going to work then setgroups() should also work.
+   */
+  if (setgroups (0, NULL) < 0)
+    _dbus_warn ("Failed to drop supplementary groups: %s\n",
+                _dbus_strerror (errno));
+  
   /* Set GID first, or the setuid may remove our permission
    * to change the GID
    */
@@ -3322,6 +3374,58 @@ _dbus_set_signal_handler (int               sig,
   sigaction (sig,  &act, 0);
 }
 
+/** Checks if a file exists
+*
+* @param file full path to the file
+* @returns #TRUE if file exists
+*/
+dbus_bool_t 
+_dbus_file_exists (const char *file)
+{
+  return (access (file, F_OK) == 0);
+}
+
+/** Checks if user is at the console
+*
+* @param username user to check
+* @param error return location for errors
+* @returns #TRUE is the user is at the consolei and there are no errors
+*/
+dbus_bool_t 
+_dbus_user_at_console (const char *username,
+                       DBusError  *error)
+{
+
+  DBusString f;
+  dbus_bool_t result;
+
+  result = FALSE;
+  if (!_dbus_string_init (&f))
+    {
+      _DBUS_SET_OOM (error);
+      return FALSE;
+    }
+
+  if (!_dbus_string_append (&f, DBUS_CONSOLE_DIR))
+    {
+      _DBUS_SET_OOM (error);
+      goto out;
+    }
+
+
+  if (!_dbus_string_append (&f, username))
+    {
+      _DBUS_SET_OOM (error);
+      goto out;
+    }
+
+  result = _dbus_file_exists (_dbus_string_get_const_data (&f));
+
+ out:
+  _dbus_string_free (&f);
+
+  return result;
+}
 
 #ifdef DBUS_BUILD_TESTS
 #include <stdlib.h>
@@ -3406,7 +3510,7 @@ _dbus_sysdeps_test (void)
       _dbus_warn ("Failed to parse double");
       exit (1);
     }
-  if (val != 3.5)
+  if (ABS(3.5 - val) > 1e-6)
     {
       _dbus_warn ("Failed to parse 3.5 correctly, got: %f", val);
       exit (1);
@@ -3424,9 +3528,9 @@ _dbus_sysdeps_test (void)
       _dbus_warn ("Failed to parse double");
       exit (1);
     }
-  if (val != 0xff)
+  if (ABS (0xff - val) > 1e-6)
     {
-      _dbus_warn ("Failed to parse 0xff correctly, got: %f", val);
+      _dbus_warn ("Failed to parse 0xff correctly, got: %f\n", val);
       exit (1);
     }
   if (pos != 4)
