@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2002, 2003  Red Hat Inc.
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -100,6 +100,12 @@ dbus_internal_do_not_use_run_tests (const char *test_data_dir)
 
   check_memleaks ();
 
+  printf ("%s: running server listen tests\n", "dbus-test");
+  if (!_dbus_server_test ())
+    die ("server listen");
+
+  check_memleaks ();
+
   printf ("%s: running object tree tests\n", "dbus-test");
   if (!_dbus_object_tree_test ())
     die ("object tree");
