@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2002  Red Hat Inc.
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
 #include <dbus/dbus-macros.h>
 #include <dbus/dbus-types.h>
 
-DBUS_BEGIN_DECLS;
+DBUS_BEGIN_DECLS
 
 typedef struct DBusMutex DBusMutex;
 typedef struct DBusCondVar DBusCondVar;
@@ -117,6 +117,6 @@ dbus_bool_t  dbus_threads_init         (const DBusThreadFunctions *functions);
 
 
 
-DBUS_END_DECLS;
+DBUS_END_DECLS
 
 #endif /* DBUS_THREADS_H */
