@@ -327,7 +327,7 @@ remove_watch (DBusWatch *watch,
     return; /* probably a not-enabled watch that was added */
 
   watch_fd->removed = TRUE;
-  watch_fd->watch = NULL;  
+  watch_fd->watch = NULL;
   
   dbus_source->watch_fds = g_list_remove (dbus_source->watch_fds, watch_fd);
 
@@ -409,7 +409,7 @@ timeout_toggled (DBusTimeout *timeout,
 
 static void
 free_source (GSource *source)
-{
+{  
   g_source_destroy (source);
 }
 
@@ -610,12 +610,11 @@ dbus_g_bus_get (DBusBusType     type,
     {
       dbus_set_g_error (error, &derror);
       dbus_error_free (&derror);
+      return NULL;
     }
-  else
-    {
-      /* does nothing if it's already been done */
-      dbus_connection_setup_with_g_main (connection, NULL);
-    }
+
+  /* does nothing if it's already been done */
+  dbus_connection_setup_with_g_main (connection, NULL);
 
   return DBUS_G_CONNECTION_FROM_CONNECTION (connection);
 }
