@@ -1,9 +1,9 @@
 /* -*- mode: C; c-file-style: "gnu" -*- */
 /* dbus-gobject.c Exporting a GObject remotely
  *
- * Copyright (C) 2003 Red Hat, Inc.
+ * Copyright (C) 2003, 2004 Red Hat, Inc.
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,9 +22,11 @@
  */
 
 #include <config.h>
-#include "dbus-glib.h"
+#include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
 #include "dbus-gtest.h"
 #include "dbus-gutils.h"
+#include "dbus-gvalue.h"
 #include <string.h>
 
 /**
@@ -148,78 +150,29 @@ gtype_to_dbus_type (GType type)
     }
 }
 
-static const char *
-dbus_type_to_string (int type)
+static void
+introspect_properties (GObject *object, GString *xml)
 {
-  switch (type)
-    {
-    case DBUS_TYPE_INVALID:
-      return "invalid";
-    case DBUS_TYPE_NIL:
-      return "nil";
-    case DBUS_TYPE_BOOLEAN:
-      return "boolean";
-    case DBUS_TYPE_INT32:
-      return "int32";
-    case DBUS_TYPE_UINT32:
-      return "uint32";
-    case DBUS_TYPE_DOUBLE:
-      return "double";
-    case DBUS_TYPE_STRING:
-      return "string";
-    case DBUS_TYPE_CUSTOM:
-      return "custom";
-    case DBUS_TYPE_ARRAY:
-      return "array";
-    case DBUS_TYPE_DICT:
-      return "dict";
-    default:
-      return "unknown";
-    }
-}
-
-static DBusHandlerResult
-handle_introspect (DBusConnection *connection,
-                   DBusMessage    *message,
-                   GObject        *object)
-{
-  GString *xml;
-  GParamSpec **specs;
-  unsigned int n_specs;
   unsigned int i;
+  unsigned int n_specs;
   GType last_type;
-  DBusMessage *ret;
-  char **path;
-  char **children;
-  
-  if (!dbus_message_get_path_decomposed (message, &path))
-    g_error ("Out of memory");
-
-  if (!dbus_connection_list_registered (connection, (const char**) path,
-                                        &children))
-    g_error ("Out of memory");
-  
-  xml = g_string_new (NULL);
-
-  g_string_append (xml, "<node>\n");
+  GParamSpec **specs;
 
   last_type = G_TYPE_INVALID;
-
   specs = g_object_class_list_properties (G_OBJECT_GET_CLASS (object),
                                           &n_specs);
 
-  i = 0;
-  while (i < n_specs)
+  for (i = 0; i < n_specs; i++ )
     {
-      GParamSpec *spec = specs[i];
-      gboolean can_set;
-      gboolean can_get;
       char *s;
       int dbus_type;
+      gboolean can_set;
+      gboolean can_get;
+      GParamSpec *spec = specs[i];
       
       dbus_type = gtype_to_dbus_type (G_PARAM_SPEC_VALUE_TYPE (spec));
       if (dbus_type == DBUS_TYPE_INVALID)
-        goto next;
+	continue;
       
       if (spec->owner_type != last_type)
 	{
@@ -231,7 +184,6 @@ handle_introspect (DBusConnection *connection,
            * general?  should people be able to set it for their
            * objects?
            */
-          
           g_string_append (xml, "  <interface name=\"org.gtk.objects.");
           g_string_append (xml, g_type_name (spec->owner_type));
           g_string_append (xml, "\">\n");
@@ -240,8 +192,8 @@ handle_introspect (DBusConnection *connection,
 	}
 
       can_set = ((spec->flags & G_PARAM_WRITABLE) != 0 &&
-                    (spec->flags & G_PARAM_CONSTRUCT_ONLY) == 0);
-
+		 (spec->flags & G_PARAM_CONSTRUCT_ONLY) == 0);
+      
       can_get = (spec->flags & G_PARAM_READABLE) != 0;
 
       s = uscore_to_wincaps (spec->name);
@@ -253,7 +205,7 @@ handle_introspect (DBusConnection *connection,
           g_string_append (xml, "\">\n");
           
           g_string_append (xml, "      <arg type=\"");
-          g_string_append (xml, dbus_type_to_string (dbus_type));
+          g_string_append (xml, _dbus_gutils_type_to_string (dbus_type));
           g_string_append (xml, "\"/>\n");
         }
 
@@ -264,33 +216,91 @@ handle_introspect (DBusConnection *connection,
           g_string_append (xml, "\">\n");
           
           g_string_append (xml, "      <arg type=\"");
-          g_string_append (xml, dbus_type_to_string (dbus_type));
+          g_string_append (xml, _dbus_gutils_type_to_string (dbus_type));
           g_string_append (xml, "\" direction=\"out\"/>\n");
         }
 
       g_free (s);
-
-    next:
-      ++i;
     }
 
   if (last_type != G_TYPE_INVALID)
     g_string_append (xml, "  </interface>\n");
 
   g_free (specs);
+}
+
+static void
+introspect_signals (GType type, GString *xml)
+{
+  guint i;
+  guint *ids, n_ids;
+
+  ids = g_signal_list_ids (type, &n_ids);
+  if (!n_ids)
+    return;
+
+  g_string_append (xml, "  <interface name=\"org.gtk.objects.");
+  g_string_append (xml, g_type_name (type));
+  g_string_append (xml, "\">\n");
+
+  /* FIXME: recurse to parent types ? */
+  for (i = 0; i < n_ids; i++)
+    {
+      guint arg;
+      GSignalQuery query;
+      
+      g_signal_query (ids[i], &query);
+
+      if (query.return_type)
+	continue; /* FIXME: these could be listed as methods ? */
+
+      g_string_append (xml, "    <signal name=\"");
+      g_string_append (xml, query.signal_name);
+      g_string_append (xml, "\">\n");
+
+      for (arg = 0; arg < query.n_params; arg++)
+	{
+	  int dbus_type = gtype_to_dbus_type (query.param_types[arg]);
+
+          g_string_append (xml, "      <arg type=\"");
+          g_string_append (xml, _dbus_gutils_type_to_string (dbus_type));
+          g_string_append (xml, "\"/>\n");
+	}
+
+      g_string_append (xml, "    </signal>\n");
+    }
+
+  g_string_append (xml, "  </interface>\n");
+}
+
+static DBusHandlerResult
+handle_introspect (DBusConnection *connection,
+                   DBusMessage    *message,
+                   GObject        *object)
+{
+  GString *xml;
+  unsigned int i;
+  DBusMessage *ret;
+  char **children;
+  
+  if (!dbus_connection_list_registered (connection, 
+                                        dbus_message_get_path (message),
+                                        &children))
+    g_error ("Out of memory");
+  
+  xml = g_string_new (NULL);
+
+  introspect_signals (G_OBJECT_TYPE (object), xml);
+  introspect_properties (object, xml);
+	  
+  g_string_append (xml, "<node>\n");
 
   /* Append child nodes */
-  
-  i = 0;
-  while (children[i])
-    {
+  for (i = 0; children[i]; i++)
       g_string_append_printf (xml, "  <node name=\"%s\"/>\n",
                               children[i]);
-      ++i;
-    }
   
   /* Close the XML, and send it to the requesting app */
-
   g_string_append (xml, "</node>\n");
 
   ret = dbus_message_new_method_return (message);
@@ -306,7 +316,6 @@ handle_introspect (DBusConnection *connection,
 
   g_string_free (xml, TRUE);
 
-  dbus_free_string_array (path);
   dbus_free_string_array (children);
   
   return DBUS_HANDLER_RESULT_HANDLED;
@@ -318,137 +327,28 @@ set_object_property (DBusConnection *connection,
                      GObject        *object,
                      GParamSpec     *pspec)
 {
-  GValue value;
-  DBusMessageIter iter;
-  int type;
-  gboolean can_set;
+  GValue value = { 0, };
   DBusMessage *ret;
+  DBusMessageIter iter;
 
   dbus_message_iter_init (message, &iter);
-  type = dbus_message_get_type (message);
-
-  can_set = TRUE;
-  switch (type)
-    {
-    case DBUS_TYPE_BYTE:
-      {
-        unsigned char b;
-
-        b = dbus_message_iter_get_byte (&iter);
-
-        g_value_init (&value, G_TYPE_UCHAR);
-
-        g_value_set_uchar (&value, b);
-      }
-      break;
-    case DBUS_TYPE_BOOLEAN:
-      {
-        gboolean b;
-
-        b = dbus_message_iter_get_boolean (&iter);
-
-        g_value_init (&value, G_TYPE_BOOLEAN);
-
-        g_value_set_boolean (&value, b);
-      }
-      break;
-    case DBUS_TYPE_INT32:
-      {
-        gint32 i;
-
-        i = dbus_message_iter_get_int32 (&iter);
-
-        g_value_init (&value, G_TYPE_INT);
-
-        g_value_set_int (&value, i);
-      }
-      break;
-    case DBUS_TYPE_UINT32:
-      {
-        guint32 i;
-
-        i = dbus_message_iter_get_uint32 (&iter);
-
-        g_value_init (&value, G_TYPE_UINT);
-
-        g_value_set_uint (&value, i);
-      }
-      break;
-    case DBUS_TYPE_INT64:
-      {
-        gint64 i;
-
-        i = dbus_message_iter_get_int64 (&iter);
-
-        g_value_init (&value, G_TYPE_INT64);
-
-        g_value_set_int64 (&value, i);
-      }
-      break;
-    case DBUS_TYPE_UINT64:
-      {
-        guint64 i;
-
-        i = dbus_message_iter_get_uint64 (&iter);
-
-        g_value_init (&value, G_TYPE_UINT64);
-
-        g_value_set_uint64 (&value, i);
-      }
-      break;
-    case DBUS_TYPE_DOUBLE:
-      {
-        double d;
-
-        d = dbus_message_iter_get_double (&iter);
-
-        g_value_init (&value, G_TYPE_DOUBLE);
-
-        g_value_set_double (&value, d);
-      }
-      break;
-    case DBUS_TYPE_STRING:
-      {
-        char *s;
-
-        /* FIXME use a const string accessor */
-
-        s = dbus_message_iter_get_string (&iter);
-
-        g_value_init (&value, G_TYPE_STRING);
-
-        g_value_set_string (&value, s);
-
-        g_free (s);
-      }
-      break;
-
-      /* FIXME array and other types, especially byte array
-       * converted to G_TYPE_STRING
-       */
-
-    default:
-      can_set = FALSE;
-      break;
-    }
 
   /* The g_object_set_property() will transform some types, e.g. it
    * will let you use a uchar to set an int property etc. Note that
    * any error in value range or value conversion will just
    * g_warning(). These GObject skels are not for secure applications.
    */
-
-  if (can_set)
+  if (dbus_gvalue_demarshal (&iter, &value))
     {
       g_object_set_property (object,
                              pspec->name,
                              &value);
 
+      g_value_unset (&value);
+
       ret = dbus_message_new_method_return (message);
       if (ret == NULL)
         g_error ("out of memory");
-
-      g_value_unset (&value);
     }
   else
     {
@@ -469,9 +369,8 @@ get_object_property (DBusConnection *connection,
                      GParamSpec     *pspec)
 {
   GType value_type;
-  gboolean can_get;
-  DBusMessage *ret;
   GValue value;
+  DBusMessage *ret;
   DBusMessageIter iter;
 
   value_type = G_PARAM_SPEC_VALUE_TYPE (pspec);
@@ -480,76 +379,14 @@ get_object_property (DBusConnection *connection,
   if (ret == NULL)
     g_error ("out of memory");
 
-  can_get = TRUE;
   g_value_init (&value, value_type);
   g_object_get_property (object, pspec->name, &value);
 
   value_type = G_VALUE_TYPE (&value);
 
   dbus_message_append_iter_init (message, &iter);
-  
-  switch (value_type)
-    {
-    case G_TYPE_CHAR:
-      dbus_message_iter_append_byte (&iter,
-                                     g_value_get_char (&value));
-      break;
-    case G_TYPE_UCHAR:
-      dbus_message_iter_append_byte (&iter,
-                                     g_value_get_uchar (&value));
-      break;
-    case G_TYPE_BOOLEAN:
-      dbus_message_iter_append_boolean (&iter,
-                                        g_value_get_boolean (&value));
-      break;
-    case G_TYPE_INT:
-      dbus_message_iter_append_int32 (&iter,
-                                      g_value_get_int (&value));
-      break;
-    case G_TYPE_UINT:
-      dbus_message_iter_append_uint32 (&iter,
-                                       g_value_get_uint (&value));
-      break;
-      /* long gets cut to 32 bits so the remote API is consistent
-       * on all architectures
-       */
-    case G_TYPE_LONG:
-      dbus_message_iter_append_int32 (&iter,
-                                      g_value_get_long (&value));
-      break;
-    case G_TYPE_ULONG:
-      dbus_message_iter_append_uint32 (&iter,
-                                       g_value_get_ulong (&value));
-      break;
-    case G_TYPE_INT64:
-      dbus_message_iter_append_int64 (&iter,
-                                      g_value_get_int64 (&value));
-      break;
-    case G_TYPE_UINT64:
-      dbus_message_iter_append_uint64 (&iter,
-                                       g_value_get_uint64 (&value));
-      break;
-    case G_TYPE_FLOAT:
-      dbus_message_iter_append_double (&iter,
-                                       g_value_get_float (&value));
-      break;
-    case G_TYPE_DOUBLE:
-      dbus_message_iter_append_double (&iter,
-                                       g_value_get_double (&value));
-      break;
-    case G_TYPE_STRING:
-      /* FIXME, the GValue string may not be valid UTF-8 */
-      dbus_message_iter_append_string (&iter,
-                                       g_value_get_string (&value));
-      break;
-    default:
-      can_get = FALSE;
-      break;
-    }
 
-  g_value_unset (&value);
-
-  if (!can_get)
+  if (!dbus_gvalue_marshal (&iter, &value))
     {
       dbus_message_unref (ret);
       ret = dbus_message_new_error (message,
@@ -666,7 +503,7 @@ static DBusObjectPathVTable gobject_dbus_vtable = {
  * class_init() for the object class.
  *
  * Once introspection information has been installed, instances of the
- * object registered with dbus_connection_register_g_object() can have
+ * object registered with dbus_g_connection_register_g_object() can have
  * their methods invoked remotely.
  *
  * @param object_class class struct of the object
@@ -704,26 +541,20 @@ dbus_g_object_class_install_info (GObjectClass          *object_class,
  * @param object the object
  */
 void
-dbus_connection_register_g_object (DBusConnection        *connection,
-                                   const char            *at_path,
-                                   GObject               *object)
+dbus_g_connection_register_g_object (DBusGConnection       *connection,
+                                     const char            *at_path,
+                                     GObject               *object)
 {
-  char **split;
-
   g_return_if_fail (connection != NULL);
   g_return_if_fail (at_path != NULL);
   g_return_if_fail (G_IS_OBJECT (object));
 
-  split = _dbus_gutils_split_path (at_path);
-
-  if (!dbus_connection_register_object_path (connection,
-                                             (const char**) split,
+  if (!dbus_connection_register_object_path (DBUS_CONNECTION_FROM_G_CONNECTION (connection),
+                                             at_path,
                                              &gobject_dbus_vtable,
                                              object))
     g_error ("Failed to register GObject with DBusConnection");
 
-  g_strfreev (split);
-
   /* FIXME set up memory management (so we break the
    * registration if object or connection vanishes)
    */
@@ -739,7 +570,7 @@ dbus_connection_register_g_object (DBusConnection        *connection,
  * Unit test for GLib GObject integration ("skeletons")
  * @returns #TRUE on success.
  */
-dbus_bool_t
+gboolean
 _dbus_gobject_test (const char *test_data_dir)
 {
   int i;
