@@ -1135,7 +1135,7 @@ dbus_g_proxy_end_call (DBusGProxy          *proxy,
   g_return_val_if_fail (pending != NULL, FALSE);
 
   dbus_pending_call_block (DBUS_PENDING_CALL_FROM_G_PENDING_CALL (pending));
-  message = dbus_pending_call_get_reply (DBUS_PENDING_CALL_FROM_G_PENDING_CALL (pending));
+  message = dbus_pending_call_steal_reply (DBUS_PENDING_CALL_FROM_G_PENDING_CALL (pending));
 
   g_assert (message != NULL);
 
@@ -1152,6 +1152,7 @@ dbus_g_proxy_end_call (DBusGProxy          *proxy,
         }
       va_end (args);
 
+      dbus_message_unref (message);
       return TRUE;
       
     case DBUS_MESSAGE_TYPE_ERROR:
@@ -1165,6 +1166,7 @@ dbus_g_proxy_end_call (DBusGProxy          *proxy,
     }
 
  error:
+  dbus_message_unref (message);
   dbus_set_g_error (error, &derror);
   dbus_error_free (&derror);
   return FALSE;
