@@ -62,6 +62,10 @@ class Bus:
 
     """bus_type=[Bus.TYPE_SESSION | Bus.TYPE_SYSTEM | Bus.TYPE_ACTIVATION]
     """
+
+    ACTIVATION_REPLY_ACTIVATED = dbus_bindings.ACTIVATION_REPLY_ACTIVATED
+    ACTIVATION_REPLY_ALREADY_ACTIVE = dbus_bindings.ACTIVATION_REPLY_ALREADY_ACTIVE 
+
     def __init__(self, bus_type=TYPE_SESSION, glib_mainloop=True):
         self._connection = dbus_bindings.bus_get(bus_type)
 
@@ -99,6 +103,10 @@ class Bus:
     def get_connection(self):
         """Get the dbus_bindings.Connection object associated with this Bus"""
         return self._connection
+    
+    def get_unix_user(self, service_name):
+        """Get the unix user for the given service_name on this Bus"""
+        return dbus_bindings.bus_get_unix_user(self._connection, service_name)
 
     def _get_match_rule(self, signal_name, interface, service, path):
         match_rule = "type='signal'"
@@ -135,6 +143,9 @@ class Bus:
             for receiver in receivers:
                 receiver(*args)
 
+    def activate_service(self, service):
+        return dbus_bindings.bus_activate_service(self._connection, service)
+
 class SystemBus(Bus):
     """The system-wide message bus
     """
