@@ -1,5 +1,5 @@
 /* -*- mode: C; c-file-style: "gnu" -*- */
-#include "dbus-glib.h"
+#include <dbus/dbus-glib.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -9,17 +9,16 @@ static GMainLoop *loop;
 int
 main (int argc, char **argv)
 {
-  DBusConnection *connection;
+  DBusGConnection *connection;
   GError *error;
-  int i;
   
   g_type_init ();
   
   loop = g_main_loop_new (NULL, FALSE);
 
   error = NULL;
-  connection = dbus_bus_get_with_g_main (DBUS_BUS_ACTIVATION,
-                                         &error);
+  connection = dbus_g_bus_get (DBUS_BUS_ACTIVATION,
+                               &error);
   if (connection == NULL)
     {
       g_printerr ("Failed to open connection to bus: %s\n",
