@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2003 Red Hat, Inc.
  *
- * Licensed under the Academic Free License version 2.0
+ * Licensed under the Academic Free License version 2.1
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 #define DBUS_TREE_VIEW_H
 
 #include <gtk/gtk.h>
-#include <glib/dbus-glib.h>
+#include <dbus/dbus-glib.h>
 #include <glib/dbus-gidl.h>
 
 GtkWidget*   dbus_tree_view_new    (void);
